//This is free and unencumbered software is provided by Cyrstalfontz as an example of
//how to program and use their line of ePaper displays. 
//
//For more information on these displays please visit: http://Crystalfontz.com
//
//Anyone is free to copy, modify, publish, use, compile, sell, or
//distribute this software, either in source code form or as a compiled
//binary, for any purpose, commercial or non - commercial, and by any
//means.
//
//In jurisdictions that recognize copyright laws, the author or authors
//of this software dedicate any and all copyright interest in the
//software to the public domain.We make this dedication for the benefit
//of the public at large and to the detriment of our heirs and
//successors.We intend this dedication to be an overt act of
//relinquishment in perpetuity of all present and future rights to this
//software under copyright law.
//
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
//MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
//OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
//ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//OTHER DEALINGS IN THE SOFTWARE.
//
//For more information, please refer to <http://unlicense.org/>

#include "ePaper.h"
const unsigned char lut_vcom0[] PROGMEM = { 0x14  ,0x01 ,0x01 ,0x05 ,0x07 ,0x05 ,0x0C ,0x0C ,0x0A ,0x04 ,0x04 ,0x0A ,0x05 ,0x07 ,0x05 };
const unsigned char lut_w[]     PROGMEM = { 0x14  ,0x01 ,0x01 ,0x45 ,0x07 ,0x05 ,0x8C ,0x4C ,0x0A ,0x84 ,0x44 ,0x0A ,0x85 ,0x07 ,0x05 };
const unsigned char lut_b[]     PROGMEM = { 0x14  ,0x01 ,0x01 ,0x05 ,0x87 ,0x05 ,0x8C ,0x4C ,0x0A ,0x84 ,0x44 ,0x0A ,0x05 ,0x47 ,0x05 };
const unsigned char lut_g1[]    PROGMEM = { 0x94  ,0x81 ,0x01 ,0x05 ,0x87 ,0x05 ,0x8C ,0x4C ,0x0A ,0x84 ,0x44 ,0x0A ,0x05 ,0x07 ,0x05 };
const unsigned char lut_g2[]    PROGMEM = { 0x94  ,0x81 ,0x01 ,0x05 ,0x87 ,0x05 ,0x8C ,0x4C ,0x0A ,0x84 ,0x44 ,0x0A ,0x05 ,0x07 ,0x05 };
const unsigned char* const luts_table[] PROGMEM = {lut_vcom0, lut_w, lut_b, lut_g1, lut_g2};
void ePaper::WriteByte(unsigned char Byte,bool data)
{
 	digitalWrite(CS, LOW);
  digitalWrite(DC, data);
  SPI.transfer(Byte);
  digitalWrite(CS, HIGH);
} 

//write a block of Flash memory to the display
void ePaper::WriteFlashImage(unsigned char cmd, const unsigned char *data, int length, bool inverted = false)
{
  int i;
  unsigned char temp;
  WriteByte(cmd,CMD);
  digitalWrite(CS, LOW);
  digitalWrite(DC, DATA);
  for( i = 0; i < length; i++)
  {

    temp = pgm_read_byte(&data[i]);
    if(inverted){
      temp = ~temp;
    }
    SPI.transfer(temp);
  }
  DataStop();
  digitalWrite(CS, HIGH);  
}

void ePaper::WriteDataBlock(unsigned char *data, int length)
{
  int i;
  digitalWrite(CS, LOW);
  digitalWrite(DC, DATA);
  for(i = 0; i < length; i++)
  {
    SPI.transfer(data[i]);
  }
 }


void ePaper::config_Pins(int Busy_Pin, int Reset_Pin, int CS_Pin,int DC_Pin, int BusSel_Pin)
{
    BUSY = Busy_Pin;
    RESET = Reset_Pin;
    CS = CS_Pin;
    DC = DC_Pin;
    BUSSEL = BusSel_Pin;
   
    pinMode(BUSY, INPUT);
    pinMode(RESET , OUTPUT);   
    pinMode(CS, OUTPUT);     
    pinMode(DC, OUTPUT);     
    pinMode(BUSSEL, OUTPUT);

    digitalWrite(RESET, HIGH);   //active low
    digitalWrite(CS, HIGH);      //active low
    digitalWrite(DC, HIGH);      //active low 
    digitalWrite(BUSSEL, LOW);   // 4 wire mode
}
  

void ePaper::SendCMD(unsigned char cmd,unsigned char * data, int length) 
{
	  WriteByte(cmd,CMD);
	  WriteDataBlock(data,length);
}

void ePaper::DownloadImage(unsigned char cmd,unsigned char * data, int length)
{
    SendCMD(cmd,data,length);
    WriteByte(DATA_STOP,CMD);
}
 
void ePaper::Reset(void) 
{
     digitalWrite(RESET, LOW);   //active low
     delay(100);
     digitalWrite(RESET, HIGH); 
     delay(1000); 
}

//ePaper::CheckStatus
//keep checking the status of the Busy_Pin till either it indicates not busy
// or we timeout waiting for it.
bool ePaper::CheckStatus(void)
{
  int timeout = 0;
 
  while(timeout < 2500)  // try a max of 25 seconds and then give up 
  {
    if(digitalRead(BUSY) == HIGH)
    {
       return true;
    }
       delay(10);    //wait 10 ms and try again
       timeout++;
  }
  return false;
}

void ePaper::LoadLUTs(void)
{
 int i,j;
 // A list of command bytes used for the individual LUT entries
  unsigned char command[] = {0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27};
  unsigned char buffer[15];
  
  for(i = 0; i < sizeof(luts_table)/sizeof(unsigned char *); i++)
  { 
     memcpy_P(buffer, (unsigned char*)pgm_read_word(&(luts_table[i])),sizeof(buffer));
     CheckStatus();      
     SendCMD(command[i],buffer,sizeof(buffer));
  }
}

